#
# Copyright 2007 Fedora Unity
#
# Jonathan Steffan <jon a fedoraunity.org>
# Jeroen van Meeuwen <kanarip a fedoraunity.org>
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; version 2 only
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Library General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
#

import os
import pykickstart
import sys

import pykickstart
import pykickstart.constants as constants
import pykickstart.parser as parser

class RevisorKickstart:
    """A robust kickstart object interfacing between Revisor and different pykickstart APIs."""
    def __init__(self, cfg):
        self.cfg = cfg

    def _reset(self):
        self.ksparser._reset()

    def create_parser(self):
        """Returns a regular kickstart parser"""
        try:
            import pykickstart.version
            self.kshandler = pykickstart.version.makeVersion(pykickstart.version.DEVEL)
            self.ksparser = pykickstart.parser.KickstartParser(self.kshandler, missingIncludeIsFatal=False)
        except:
            import pykickstart.data
            self.ksdata = pykickstart.data.KickstartData()
            self.kshandler = pykickstart.parser.KickstartHandlers(self.ksdata)
            self.ksparser = pykickstart.parser.KickstartParser(self.ksdata, self.kshandler, missingIncludeIsFatal=False)

    def _kshandler(self):
        """Returns the kickstart data object instance if it's there,
        and if it's not, it returns the handler"""
        if hasattr(self,"ksdata"):
            return self.ksdata
        else:
            return self.kshandler

    def read_file(self, fn):
        """Reads in a kickstart file and returns True on success or False on failure"""
        self.ksparser.readKickstart(fn)

    def _reset(self):
        self.ksparser._reset()

    def __str__(self):
        if hasattr(self,"ksdata"):
            kswriter = pykickstart.writer.KickstartWriter(self.ksdata)
            return kswriter.write()
        else:
            return self.kshandler.__str__()

    def _get(self, item=None, val=None):
        # pykickstart >= 0.100
        if not item == None:
            if hasattr(self,"ksdata"):
                self_ref = self.ksdata
            else:
                self_ref = self.kshandler

            if hasattr(self_ref,item):
                if not val == None:
                    if hasattr(getattr(self_ref,item),val):
                        return getattr(getattr(self_ref,item),val)
                    elif isinstance(getattr(self_ref,item), dict):
                        return getattr(self_ref,item)[val]
                    else:
                        pass
                else:
                    return getattr(self_ref,item)
            elif hasattr(self_ref,val):
                return getattr(self_ref,val)

        elif hasattr(self,"ksdata"):
            return self.ksdata
        else:
            return self.kshandler

    def _set(self, item1, item2=None, val=None):
        # ks.item1 = val
        # ks.item1.item2 = val
        # ks.item1[item2] = val

        if hasattr(self,"ksdata"):
            self_ref = self.ksdata
        else:
            self_ref = self.kshandler

        if hasattr(self_ref,item1):
            if not item2 == None:
                self_ref = getattr(self_ref,item1)
                if hasattr(self_ref,item2):
                    setattr(self_ref,item2,val)
                elif isinstance(getattr(self_ref,item2), dict):
                    self_ref[item2] = val
                else:
                    raise AttributeError
            else:
                setattr(self_ref,item1,val)
        else:
            raise AttributeError

    def _NetworkData(self):
        if hasattr(self,ksdata):
            ks_net_data = pykickstart.data.KickstartNetworkData()
        else:
            ks_net_data = self.kshandler.NetworkData()
        return ks_net_data
