#
# Copyright 2007 Fedora Unity
#
# Jonathan Steffan <jon a fedoraunity.org>
# Jeroen van Meeuwen <kanarip a fedoraunity.org>
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; version 2 only
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Library General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

import sys
import pykickstart
import tempfile
import time
import traceback
import subprocess
import shutil
import yum

from revisor import pilgrim
#from revisor import build_media
import revisor.progress
#from revisor import lm_install_callback
from revisor.errors import *

# Import constants
from revisor.constants import *

# Translation
from rhpl.translate import _, N_, getDefaultLangs
import rhpl.translate as translate

class LMOptical(pilgrim.InstallationTarget):
    def __init__(self, base):
        self.gui = base.mode
        self.base = base
        self.log = base.log
        self.cfg = base.cfg

        if self.cfg.revisorUseGUI:
            import gtk
            import gtk.glade
            import gobject
            import gtk.gdk as gdk

    def installPackages(self, pbar = None):
        """install packages into target file system"""

        self.cfg.check_package_selection()

        self.runInstall(pbar = pbar)
        return True

    def transactionErrors(self, errs):
        self.log.error(_("""Error encountered during installation of the software you selected:

%s""") % (errs), recoverable = False)

    def runInstall(self, pbar = None):

        self.log.debug(_("Running package installation"))

        (res, resmsg) = self.cfg.yumobj.buildTransaction()
        if res != 2:
            msg = _("Unable to build transaction")
            for m in resmsg:
                msg = "%s %s" % (msg,m)
            self.log.error(msg, recoverable = False)
        else:
            self.log.debug(_("Succesfully built transaction: ret %s, msg %s") % (res, ' '.join(resmsg)))

        self.cfg.yumobj.ts.check()
        self.cfg.yumobj.ts.order()

        if not pbar: pbar = self.base.progress_bar(_("Installing Software to Live Media"))

        if self.cfg.revisorUseGUI:
            tsprog = revisor.progress.TransactionProgressGUI(pbar, log = self.log)
        else:
            tsprog = revisor.progress.TransactionProgressCLI(pbar, log = self.log)

        del self.cfg.yumobj.ts
        self.cfg.yumobj.initActionTs() # make a new, blank ts to populate
        self.cfg.yumobj.populateTs(keepold=0)
        self.cfg.yumobj.ts.check() #required for ordering
        self.cfg.yumobj.ts.order() # order

        tsprog.tsInfo = self.cfg.yumobj.tsInfo

        try:
            tserrors = self.cfg.yumobj.runTransaction(tsprog)

        except yum.Errors.YumBaseError, err:
            self.log.debug("yum.Errors.YumBaseError: %s" % err)
            pbar.destroy()
            self.transactionErrors(err)

        except RevisorError, err:
            self.log.debug("RevisorError: %s" % err)
            pbar.destroy()
            self.transactionErrors(err)

        pbar.destroy()

    def extra_configuration(self):
# Live Media User Configuration
        instroot = "%s/install_root" %(self.build_dir,)
        if self.cfg.lm_user_configuration:
            subprocess.call(["/usr/sbin/useradd", "-c", self.cfg.lm_user_comment, self.cfg.lm_user_name], preexec_fn=self.run_in_root)
            p1 = subprocess.Popen(["/bin/echo", self.cfg.lm_user_password], stdout=PIPE, preexec_fn=self.run_in_root)
            p2 = subprocess.Popen(["/usr/bin/passwd", "--stdin", self.cfg.lm_user_name], stdin=p1.stdout, stdout=PIPE, preexec_fn=self.run_in_root)
            output = p2.communicate()[0]
            print output
            if self.cfg.lm_user_wheel:
                subprocess.call(["/usr/bin/gpasswd", "-a", self.cfg.lm_user_name, "wheel"], preexec_fn=self.run_in_root)
            if self.cfg.lm_wheel_sudo_no_passwd:
                subprocess.call(["/bin/sed", "-i", "-e", "s/^# %wheel.*ALL=(ALL).*NOPASSWD: ALL.*/%wheel ALL=(ALL) NOPASSWD: ALL/g", "/etc/sudoers"], preexec_fn=self.run_in_root)

            # Also have root's mail forwarded to this user
            subprocess.call(["/bin/sed", "-i", "-e", "s/^#root:.*marc.*/root: " + self.cfg.lm_user_name + "/g", "/etc/aliases"], preexec_fn=self.run_in_root)


        self.log.debug(_("Xorg is installed: %s") % (self.cfg.package_Xorg))
        # Init 5 if we have Xorg and it is enabled (Enabled Default: True; Installed Default: False)
        if self.cfg.ksobj._get("xconfig","startX") and self.cfg.package_Xorg:
            f = open("%s/etc/inittab" %(instroot,), "rw+")
            buf = f.read()
            buf = buf.replace("id:3:initdefault", "id:5:initdefault")
            f.seek(0)
            f.write(buf)
            f.close()
        else:
            f = open("%s/etc/inittab" %(instroot,), "rw+")
            buf = f.read()
            buf = buf.replace("id:5:initdefault", "id:3:initdefault")
            f.seek(0)
            f.write(buf)
            f.close()

        self.log.debug(_("Default desktop is %s") % (self.cfg.ksobj._get("xconfig","defaultdesktop")))
        self.log.debug(_("Gnome is installed: %s") % (self.cfg.package_windowmanager_gnome))
        self.log.debug(_("KDE is installed: %s") % (self.cfg.package_windowmanager_kde))
        self.log.debug(_("XFCE is installed: %s") % (self.cfg.package_windowmanager_xfce))

        # Setup our default desktop
        if len(self.cfg.ksobj._get("xconfig","defaultdesktop")) > 0:
            f = open("%s/etc/sysconfig/desktop" %(instroot,), "w")
            buf = "DESKTOP=%s\n" % (self.cfg.ksobj._get("xconfig","defaultdesktop"))
            f.write(buf)
            f.close()
        else: # At least set something? (Default: GNOME)
            f = open("%s/etc/sysconfig/desktop" %(instroot,), "w")
            buf = "DESKTOP=GNOME\n"
            f.write(buf)
            f.close()

        # Setup Xorg.conf
        if self.cfg.package_Xorg:
            keyboard = "us"
            driver = "vesa"
            if len(self.cfg.ksobj._get("keyboard","keyboard")) > 0: keyboard = self.cfg.ksobj._get("keyboard","keyboard")
            if len(self.cfg.ksobj._get("xconfig","driver")) > 0: driver = self.cfg.ksobj._get("xconfig","driver")
            f = open("%s/etc/X11/xorg.conf" %(instroot,), "w")
            xorg_config = self.cfg.default_xorg_config % (keyboard, driver)
            f.write(xorg_config)
            f.close()

    def createIso(self, callback):
        """write out the live CD ISO"""
        # Check if the destination directory is there
        if not os.access(os.path.join(self.cfg.destination_directory,"live"), os.R_OK):
            os.makedirs(os.path.join(self.cfg.destination_directory,"live"))

        self.callback = callback

        # Just disabled so I can actually do a screencast

# Hey this is a release candidate let's enable this code, and...
        self.base.run_command(["/usr/bin/mkisofs", "-o", os.path.join(self.cfg.destination_directory,"live","%s.iso" % self.fs_label),
                               "-b", "isolinux/isolinux.bin",
                               "-c", "isolinux/boot.cat",
                               "-no-emul-boot", "-boot-load-size", "4",
                               "-boot-info-table", "-J", "-r", "-hide-rr-moved",
                               "-V", "%s" % self.fs_label, "%s/out" % self.build_dir])

# Disable the old code. Genius!
#
#        subprocess.call(["/usr/bin/mkisofs", "-o", "%s.iso" % os.path.join(self.cfg.destination_directory,"live",self.fs_label),
#                         "-b", "isolinux/isolinux.bin",
#                         "-c", "isolinux/boot.cat",
#                         "-no-emul-boot", "-boot-load-size", "4",
#                         "-boot-info-table", "-J", "-r", "-hide-rr-moved",
#                         "-V", "%s" % self.fs_label, "%s/out" % (self.build_dir)])

        # implant an isomd5sum
        if os.path.exists("/usr/lib/anaconda-runtime/implantisomd5"):
            subprocess.call(["/usr/lib/anaconda-runtime/implantisomd5",
                             "%s.iso" % os.path.join(self.cfg.destination_directory,"live",self.fs_label)])
        else:
            self.log.warning(_("anaconda-runtime not installed; not setting up mediacheck"))

    def createSquashFS(self, callback):
        """create compressed squashfs file system"""
# FIXME:
# mksquashfs segfaults if PWD isn't set in the environment
#        self.base.run_command(["/sbin/mksquashfs", "%s/data/os.img" % (self.build_dir), "%s/data/sysroot" % (self.build_dir),
#                               "%s/out/squashfs.img" % (self.build_dir)],
#                               inshell=True,
#                               rundir="%s/data" % (self.build_dir),
#                               env={"PWD": "%s/data" % (self.build_dir)})
        subprocess.call(["/sbin/mksquashfs", "os.img", "sysroot",
                             "../out/squashfs.img"],
#                             stdout=subprocess.PIPE,
#                             stderr=subprocess.STDOUT,
                             cwd="%s/data" %(self.build_dir,),
                             env={"PWD": "%s/data" %(self.build_dir,)})
